type AnyFn = (...args: any[]) => any;
type InstrumentFunction = <X extends AnyFn>(name: string, fn: X) => X;

type Provider<Provide, Input = unknown> = (() => Provide | Promise<Provide>) | ((input: Input) => Provide | Promise<Provide>);
type ProviderOption = {
    instrument?: InstrumentFunction;
    mode: 'prototype' | 'singleton';
};
declare const defaultProviderOptions: ProviderOption;
type Executor<Provide> = {
    execute<Output>(executable: Provider<Output, Provide>): Promise<Output>;
    get(): Promise<Provide>;
    prepare<Input>(provider: (provide: Provide, input: Input) => any): (input: Input) => Promise<Awaited<ReturnType<typeof provider>>>;
};
type Hijacked<Dependent> = (executor: Executor<Dependent>) => void;
declare function create<Provide, Dependent = unknown>(provider: Provider<Provide, Dependent>, dependent?: Executor<Dependent>, options?: ProviderOption): {
    execute: <Output>(provider: Provider<Output, Provide>) => Promise<Output>;
    get: () => Promise<Provide>;
    prepare: <Input>(provider: (provide: Provide, input: Input) => any) => (input: Input) => Promise<any>;
    _inject: Hijacked<Dependent>;
};
declare const createProvider: <Output, Input>(provider: Provider<Output, Input>) => Provider<Output, Input>;
declare const value: <Provide>(value: Provide) => {
    execute: <Output>(provider: Provider<Output, Provide>) => Promise<Output>;
    get: () => Promise<Provide>;
    prepare: <Input>(provider: (provide: Provide, input: Input) => any) => (input: Input) => Promise<any>;
    _inject: Hijacked<unknown>;
};
declare const from: <Dependent>(executor: Executor<Dependent>) => {
    provide: <Provide>(provider: Provider<Provide, Dependent>, opts?: ProviderOption) => {
        execute: <Output>(provider: Provider<Output, Provide>) => Promise<Output>;
        get: () => Promise<Provide>;
        prepare: <Input>(provider: (provide: Provide, input: Input) => any) => (input: Input) => Promise<any>;
        _inject: Hijacked<Dependent>;
    };
    execute: <Output_1>(executable: Provider<Output_1, Dependent>) => Promise<Output_1>;
    prepare: <Input_1>(provider: (provide: Dependent, input: Input_1) => any) => (input: Input_1) => Promise<any>;
};
type inferProvide<T> = T extends Executor<infer S> ? S : never;
declare const combine: <L extends Record<string, Executor<any>>>(layout: L) => Executor<{ [key in keyof L]: inferProvide<L[key]>; }>;

export { Executor, Hijacked, ProviderOption, combine, create, createProvider, defaultProviderOptions, from, inferProvide, value };
